import logging

from threading import Thread

from django import shortcuts
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login as auth_login, logout as auth_logout
from django.contrib.auth.decorators import login_required
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt

try:
    from django.utils.http import is_safe_url
except ImportError:
    from .utils import is_safe_url

from keystoneclient.v2_0 import client as keystone_client
from keystoneclient import exceptions as keystone_exceptions

from .forms import Login
from .user import set_session_from_user, create_user_from_token
from .exceptions import KeystoneAuthException

from django.http import HttpResponse, HttpResponseForbidden

LOG = logging.getLogger(__name__)

@sensitive_post_parameters()
@csrf_exempt
@never_cache
def login(request):
    if request.method == 'POST':
        username = request.POST.get('username')
        password = request.POST.get('password')
        region = request.POST.get('region')
        tenant = request.POST.get('tenant')

        if not tenant:
            tenant = None

        try:
            user = authenticate(request=request,
                                username=username,
                                password=password,
                                tenant=tenant,
                                auth_url=region)
            LOG.warning( 'Login successful for user "%s".' % username )
        except KeystoneAuthException as exc:
            LOG.warning( 'Login failed for user "%s".' % username )
            request.session.flush()
            return HttpResponseForbidden('Incorrect login data.')
        
        auth_login(request, user)
        if request.user.is_authenticated():
            set_session_from_user(request, request.user)
            request.session['region_endpoint'] = request.user.endpoint
            request.session['region_name'] = "Default Region"
        return shortcuts.redirect( settings.LOGIN_REDIRECT_URL + 'project' )

    return shortcuts.redirect( settings.SSP_LOGOUT_URL )


def logout(request):
    LOG.warning( 'Logging out user "%s".' % request.user.username )
    if 'token_list' in request.session:
        t = Thread(target=delete_all_tokens,
                   args=(list(request.session['token_list']),))
        t.start()
    """ Securely logs a user out. """
    auth_logout(request)
    return shortcuts.redirect( settings.SSP_LOGOUT_URL )


def delete_all_tokens(token_list):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    for token_tuple in token_list:
        try:
            endpoint = token_tuple[0]
            token = token_tuple[1]
            client = keystone_client.Client(endpoint=endpoint,
                                            token=token,
                                            insecure=insecure)
            client.tokens.delete(token=token)
        except keystone_exceptions.ClientException as e:
            LOG.info('Could not delete token')


@login_required
def switch(request, tenant_id, redirect_field_name=REDIRECT_FIELD_NAME):
    """ Switches an authenticated user from one tenant to another. """
    LOG.debug('Switching to tenant %s for user "%s".'
              % (tenant_id, request.user.username))
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    endpoint = request.user.endpoint
    client = keystone_client.Client(endpoint=endpoint,
                                    insecure=insecure)
    try:
        token = client.tokens.authenticate(tenant_id=tenant_id,
                                        token=request.user.token.id)
        msg = 'Tenant switch successful for user "%(username)s".' % \
            {'username': request.user.username}
        LOG.info(msg)
    except keystone_exceptions.ClientException:
        msg = 'Tenant switch failed for user "%(username)s".' % \
            {'username': request.user.username}
        LOG.warning(msg)
        token = None
        LOG.exception('An error occurred while switching sessions.')

    # Ensure the user-originating redirection url is safe.
    # Taken from django.contrib.auth.views.login()
    redirect_to = request.REQUEST.get(redirect_field_name, '')
    if not is_safe_url(url=redirect_to, host=request.get_host()):
        redirect_to = settings.LOGIN_REDIRECT_URL

    if token:
        user = create_user_from_token(request, token, endpoint)
        set_session_from_user(request, user)
    return shortcuts.redirect(redirect_to)


def switch_region(request, region_name,
                  redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Switches the non-identity services region that is being managed
    for the scoped project.
    """
    if region_name in request.user.available_services_regions:
        request.session['services_region'] = region_name
        LOG.debug('Switching services region to %s for user "%s".'
                  % (region_name, request.user.username))

    redirect_to = request.REQUEST.get(redirect_field_name, '')
    if not is_safe_url(url=redirect_to, host=request.get_host()):
        redirect_to = settings.LOGIN_REDIRECT_URL

    return shortcuts.redirect(redirect_to)
